@@ -16,7 +16,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
-# $Id: Iterator.pm,v 1.13 2005/10/23 16:31:15 dan Exp $
+# $Id: Iterator.pm,v 1.14 2005/11/21 11:39:51 dan Exp $
 
 =pod
 
@@ -321,9 +321,14 @@ sub append {
     my $value = shift;
     my $type = shift;
     
+    if (ref($value) eq "Net::DBus::Binding::Value") {
+	$type = $value->type;
+	$value = $value->value;
+    }
+
     if (!defined $type) {
 	$type = $self->guess_type($value);
-    }
+    }	
 
     if (ref($type) eq "ARRAY") {
 	my $maintype = $type->[0];
@@ -338,7 +343,10 @@ sub append {
 	} else {
 	    confess "Unsupported compound type ", $maintype;
 	}
-    } else {
+    } else {	
+	# XXX is this good idea or not
+	$value = '' unless defined $value;
+
 	if ($type == &Net::DBus::Binding::Message::TYPE_BOOLEAN) {
 	    $self->append_boolean($value);
 	} elsif ($type == &Net::DBus::Binding::Message::TYPE_BYTE) {
@@ -513,11 +521,8 @@ L<Net::DBus::Binding::Message>
 
 Daniel Berrange E<lt>dan@berrange.comE<gt>
 
-=head1 COPYRIGHT AND LICENSE
+=head1 COPYRIGHT
 
 Copyright 2004 by Daniel Berrange
 
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself. 
-
 =cut
